<?php
/**
 * Rakim backend implementation for PHP's PEAR database abstraction
 * layer.
 *
 * Required values for $params:<pre>
 *      'phptype'       The database type (e.g. 'pgsql', 'mysql', etc.).
 *      'hostspec'      The hostname of the database server.
 *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
 *      'username'      The username with which to connect to the database.
 *      'password'      The password associated with 'username'.
 *      'database'      The name of the database.
 *      'charset'       The database's internal charset.
 *
 * Required by some database implementations:
 *      'options'       Additional options to pass to the database.
 *      'tty'           The TTY on which to connect to the database.
 *      'port'          The port on which to connect to the database.
 *
 * The table structure can be created by the scripts/drivers/rakim.sql
 * script.
 *
 * $Horde: rakim/lib/Driver/sql.php,v 1.14 2004/10/08 15:45:26 chuck Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.14 $
 * @since   Rakim 0.1
 * @package Rakim
 */
class Rakim_Driver_sql extends Rakim_Driver {

    /** Hash containing connection parameters. */
    var $_params = array();

    /** Handle for the current database connection.
        @var object DB $_db */
    var $_db;

    /** Boolean indicating whether or not we're connected to the SQL server. */
    var $_connected = false;

    /**
     * Constructs a new SQL Rakim backend object.
     *
     * @param array  $params    A hash containing connection parameters.
     */
    function Rakim_Driver_sql($params = null)
    {
        if (is_null($params)) {
            $params = Horde::getDriverConfig('backend', 'sql');
        }

        $this->_params = $params;
        $this->_connect();
    }

    /**
     * Update the last known active time for the current user.
     *
     * @param optional boolean $logout  If true, set the refresh_time to null,
     *                                  marking the user inactive.
     * @param optional string  $user    If different from Auth::getAuth(), the
     *                                  username to update.
     *
     * @return mixed  True on success, PEAR_Error on failure.
     */
    function updateLastActive($logout = false, $user = null)
    {
        $this->_connect();

        if ($logout) {
            $time = 'NULL';
        } else {
            $time = time();
        }

        if (empty($user)) {
            $user = Auth::getAuth();
        }

        $query = sprintf('INSERT INTO rakim_support_personnel (user_uid, user_last_active) VALUES (%s, %s)',
                         $this->_db->quote($user), $time);

        $result = $this->_db->query($query);
        if (is_a($result, 'PEAR_Error')) {
            $query = sprintf('UPDATE rakim_support_personnel SET user_last_active = %s WHERE user_uid = %s',
                             $time, $this->_db->quote($user));

            $result = $this->_db->query($query);
            if (is_a($result, 'PEAR_Error')) {
                Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            }
        }

        return $result;
    }

    /**
     * Get a list of all support personnel currently active.
     *
     * @return mixed  An array of user ids, or a PEAR_Error on failure.
     */
    function getActivePersonnel()
    {
        $this->_connect();

        $query = sprintf('SELECT user_uid, user_uid FROM rakim_support_personnel WHERE %s - user_last_active < 60', time());
        $result = $this->_db->getAssoc($query);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return array();
        }

        return array_values($result);
    }

    /**
     * List all organizations in this Rakim system.
     *
     * @return mixed  An array of organization ids, or a PEAR_Error on failure.
     */
    function getOrganizations()
    {
        $this->_connect();

        $query = 'SELECT organization_id, organization_name FROM rakim_organizations';
        $result = $this->_db->getAssoc($query);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return array();
        }

        return $result;
    }

    function getEmailResponses($id)
    {
        $query = 'SELECT * FROM rakim_email_responses WHERE email_id = ' . (int)$id . ' ORDER BY response_time';
        $result = $this->_db->getAll($query, DB_FETCHMODE_ASSOC);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return array();
        }

        return $result;
    }

    /**
     * Get an organization by id.
     *
     * @param integer $id  The organization's id.
     *
     * @return string  The organization name, or 'No organization'
     *                 if it isn't found.
     */
    function getOrganization($id)
    {
        if (empty($id)) {
            return _("No organization");
        }

        $this->_connect();
        $query = sprintf('SELECT organization_name FROM rakim_organizations WHERE organization_id = %s',
                         (int)$id);
        $result = $this->_db->getOne($query);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return _("No organization");
        }

        return $result ? $result : _("No organization");
    }

    /**
     * Get the transcript for a session.
     */
    function getTranscript($sid = null, $gid = null)
    {
        if (!is_null($sid)) {
            $getTranscriptQuery = 'SELECT m.message_id, m.message_text AS message, m.message_sender AS sender_name
                FROM rakim_messages m, rakim_guests g
                WHERE m.guest_id = g.guest_id
                      AND g.session_id = ' . $this->_db->quote($sid) . ' ORDER BY message_time';
        } elseif (!is_null($gid)) {
            $getTranscriptQuery = 'SELECT message_id, message_text AS message, message_sender AS sender_name
                FROM rakim_messages WHERE guest_id = ' . $this->_db->quote($gid) . ' ORDER BY message_time';
        } else {
            return false;
        }

        Horde::logMessage($getTranscriptQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);

        $result = $this->_db->getAssoc($getTranscriptQuery);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return false;
        }

        return $result;
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean    True on success; exits (Horde::fatal()) on error.
     */
    function _connect()
    {
        if (!$this->_connected) {
            Horde::assertDriverConfig($this->_params, 'backend',
                array('phptype', 'hostspec', 'username', 'database', 'charset'));

            /* Connect to the SQL server using the supplied parameters. */
            require_once 'DB.php';
            $this->_db = &DB::connect($this->_params,
                                      array('persistent' => !empty($this->_params['persistent'])));
            if (is_a($this->_db, 'PEAR_Error')) {
                Horde::fatal($this->_db, __FILE__, __LINE__);
            }

            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

            $this->_connected = true;
        }

        return true;
    }

}
